unit MT5DataServerUnit;

interface

uses
  SysUtils,
  Classes,
  Sockets,
  SyncObjs;

type
  TQueryItem=class
  protected
    FSocket:TCustomIpClient;
    FQuery:string;
    FReplay:string;
  public
    constructor Create(aSocket:TCustomIpClient; aQuery:string);
  end;

type
  TQueryList=class
    FCriticalSection:TCriticalSection;
    FList:TList;
    FCurrentQuery:TQueryItem;
    constructor Create();
    destructor Destroy(); override;
    procedure AddQuery(aSocket:TCustomIpClient; aQuery:string);
    procedure GetQuery(var s:string);
    procedure AddReplay(s:string);
    function GetReplay(aSocket:TCustomIpClient):TQueryItem;
    procedure RemoveAll(aSocket:TCustomIpClient);
  end;

type
  TMT5TcpServer=class(TTcpServer)
    FQueryList : TQueryList;
  protected
    procedure DoAccept(ClientSocket: TCustomIpClient); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

type
  TAnsiCharArray=array[0..255] of AnsiChar;

function StartServer(host:PWideChar; port:PWideChar) : boolean;  stdcall;
function StopServer() : boolean; stdcall;
//procedure GetQuery(var s:string); stdcall;
//procedure AddReplay(s:string); stdcall;
procedure GetQuery(wc:PWideChar); stdcall;
procedure AddReplay(wc:PWideChar); stdcall;

implementation

{ TQueryItem }

constructor TQueryItem.Create(aSocket:TCustomIpClient; aQuery:string);
begin
  inherited Create();
  FSocket:=aSocket;
  FQuery:=aQuery;
  FReplay:='';
end;

{ TQueryList }

constructor TQueryList.Create;
begin
  inherited Create();
  FCriticalSection:=TCriticalSection.Create();
  FList:=TList.Create();
end;

destructor TQueryList.Destroy;
var
  i:integer;
begin
  for i:=FList.Count-1 downto 0 do begin
    TQueryItem(FList[i]).Free;
    FList.Delete(i);
  end;
  FList.Free();
  FCriticalSection.Free();
  inherited Destroy();
end;

procedure TQueryList.AddQuery(aSocket: TCustomIpClient; aQuery: string);
var
  T:TQueryItem;
begin
  T:=TQueryItem.Create(aSocket,aQuery);
//T.FReplay:=aQuery+'OK1';
  FCriticalSection.Enter;
  try
    FList.Add(T);
  finally
    FCriticalSection.Leave;
  end;
end;

procedure TQueryList.GetQuery(var s:string);
var
  i:integer;
begin
//  Result:=0;
  s:='';
  FCriticalSection.Enter;
  try
  if (FCurrentQuery=nil) and (FList.Count>0) then begin
    i:=0;
    while (i<FList.Count) and (TQueryItem(FList[i]).FReplay<>'')
    do
      i:=i+1;
    if (i<FList.Count) then begin
      FCurrentQuery:=TQueryItem(FList[0]);
      s:=FCurrentQuery.FQuery;
    end;
//    Result:=Length(CurrentQuery.FQuery);
//    StringToWideChar(CurrentQuery.FQuery, query, Result);
  end;
  finally
    FCriticalSection.Leave;
  end;
end;

procedure TQueryList.AddReplay(s:string);
begin
  if (FCurrentQuery<>nil) then begin
    FCurrentQuery.FReplay:=s;
    FCurrentQuery:=nil;
  end;
end;

function TQueryList.GetReplay(aSocket: TCustomIpClient): TQueryItem;
var
  i:integer;
begin
  Result:=nil;
  i:=0;
  FCriticalSection.Enter;
  try
    while (i<FList.Count) and (Result=nil) do begin
      if (TQueryItem(FList[i]).FSocket=aSocket) and
         (TQueryItem(FList[i]).FReplay<>'')
      then
        Result:=TQueryItem(FList[i]);
      i:=i+1;
    end;
    if Result<>nil then begin
      FList.Delete(FList.IndexOf(Result));
    end;
  finally
    FCriticalSection.Leave;
  end;
end;

procedure TQueryList.RemoveAll(aSocket: TCustomIpClient);
var
  i:integer;
begin
  FCriticalSection.Enter;
  try
    for i:=FList.Count-1 downto 0 do begin
      if TQueryItem(FList[i]).FSocket=aSocket then begin
        TQueryItem(FList[i]).Free;
        FList.Delete(i);
      end;
    end;
  finally
    FCriticalSection.Leave;
  end;
end;

{ TMT5TcpServer }

constructor TMT5TcpServer.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FQueryList:=TQueryList.Create();
end;

destructor TMT5TcpServer.Destroy;
begin
  FQueryList.Free();
  inherited Destroy();
end;

procedure TMT5TcpServer.DoAccept(ClientSocket: TCustomIpClient);
var s:string; q:TQueryItem; buy:boolean;
begin
  buy:=FALSE;
  ClientSocket.Sendln('HELLOW MT5 DATASERVER');
  repeat
    s:='';
    if ClientSocket.Connected and ClientSocket.WaitForData(0) then begin
      s:=ClientSocket.Receiveln();
      if (s='BUY') then
        buy:=TRUE;
      if (s<>'') and not buy then
        FQueryList.AddQuery(ClientSocket,s);
    end;
    q:=FQueryList.GetReplay(ClientSocket);
    if ClientSocket.Connected and (q<>nil) then
      ClientSocket.Sendln(q.FReplay);
    if (s='') and (q=nil) then
      Sleep(100);
  until (not ClientSocket.Connected or buy);
end;

var
  TcpServer : TMT5TcpServer;

{ Export functions }

function StartServer(host:PWideChar; port:PWideChar) : boolean;  stdcall;
begin
  if not Assigned(TcpServer) then
    TcpServer:=TMT5TcpServer.Create(nil);
  with TcpServer do begin
    if Active then Close();
    LocalHost:=host;
    LocalPort:=port;
    BlockMode:=bmThreadBlocking;
    try
      Open();
    finally
      Result:=Active;
    end;
  end;
end;

function StopServer() : boolean; stdcall;
begin
  Result:=True;
  if Assigned(TcpServer) then begin
    TcpServer.Close();
    Result:=not TcpServer.Active;
    TcpServer.Free();
  end;
end;

//var
//  ReadBuffer:TAnsiCharArray;
//  BufferSize:integer=0;

{
procedure GetQuery(var s:string); stdcall;
begin
  if Assigned(TcpServer) then
    TcpServer.FQueryList.GetQuery(s);
end;

procedure AddReplay(s:string); stdcall;
begin
  if Assigned(TcpServer) then
    TcpServer.FQueryList.AddReplay(s);
end;
}

procedure GetQuery(wc:PWideChar); stdcall;
var
  s:string;
begin
  if Assigned(TcpServer) then begin
    TcpServer.FQueryList.GetQuery(s);
//    if s<>'' then wc[0]:='A';
    StringToWideChar(s,wc,1024*2);
  end;
end;

procedure AddReplay(wc:PWideChar); stdcall;
var
  s:string;
begin
  if Assigned(TcpServer) then begin
    s:=WideCharToString(wc);
    TcpServer.FQueryList.AddReplay(s);
  end;
end;

end.
